"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NightwatchInit = void 0;
const node_fs_1 = __importDefault(require("node:fs"));
const path_1 = __importDefault(require("path"));
const ejs_1 = __importDefault(require("ejs"));
const ansi_colors_1 = __importDefault(require("ansi-colors"));
const https_1 = __importDefault(require("https"));
const inquirer_1 = require("inquirer");
const child_process_1 = require("child_process");
const uuid_1 = require("uuid");
const utils_1 = require("./utils");
const logger_1 = __importDefault(require("./logger"));
const boxen_1 = __importDefault(require("boxen"));
const constants_1 = require("./constants");
const downloads_json_1 = __importDefault(require("./downloads.json"));
const defaults_json_1 = __importDefault(require("./defaults.json"));
const defaultsMobile_json_1 = __importDefault(require("./defaultsMobile.json"));
const defaultsApp_json_1 = __importDefault(require("./defaultsApp.json"));
const mobile_helper_1 = require("@nightwatch/mobile-helper");
const node_util_1 = require("node:util");
const EXAMPLE_TEST_FOLDER = 'examples';
const DEFAULT_FOLDER = 'nightwatch';
var Runner;
(function (Runner) {
    Runner["Cucumber"] = "cucumber";
    Runner["Mocha"] = "mocha";
    Runner["Js"] = "js";
})(Runner || (Runner = {}));
class NightwatchInit {
    constructor(rootDir = process.cwd(), options) {
        this.rootDir = rootDir;
        this.options = options;
        this.otherInfo = {};
        this.onlyConfig = false;
        this.client_id = (0, uuid_1.v4)();
    }
    run() {
        var _a, _b, _c, _d, _e, _f, _g;
        return __awaiter(this, void 0, void 0, function* () {
            let answers = {};
            if ((_a = this.options) === null || _a === void 0 ? void 0 : _a['generate-config']) {
                this.onlyConfig = true;
            }
            if ((_b = this.options) === null || _b === void 0 ? void 0 : _b.yes) {
                if ((_c = this.options) === null || _c === void 0 ? void 0 : _c.mobile) {
                    answers = defaultsMobile_json_1.default;
                    if ((_d = this.options) === null || _d === void 0 ? void 0 : _d.browser) {
                        answers.mobileBrowsers = this.options.browser;
                    }
                }
                else if ((_e = this.options) === null || _e === void 0 ? void 0 : _e.native) {
                    answers = defaultsApp_json_1.default;
                }
                else {
                    answers = defaults_json_1.default;
                    if ((_f = this.options) === null || _f === void 0 ? void 0 : _f.browser) {
                        answers.browsers = this.options.browser;
                    }
                }
            }
            else {
                logger_1.default.info((0, node_util_1.format)(constants_1.CONFIG_INTRO, this.rootDir));
                answers = yield this.askQuestions();
                // Add a newline after questions.
                logger_1.default.info();
            }
            this.refineAnswers(answers);
            // Install Packages
            const packagesToInstall = this.identifyPackagesToInstall(answers);
            this.installPackages(packagesToInstall);
            // Setup TypeScript
            if (!this.onlyConfig && answers.language === 'ts') {
                this.setupTypescript();
            }
            // Check if Java is installed on the system
            if (answers.seleniumServer) {
                this.checkJavaInstallation();
            }
            // Install/Update webdrivers
            const webdriversToInstall = this.identifyWebdriversToInstall(answers);
            if (webdriversToInstall.length) {
                yield this.installWebdrivers(webdriversToInstall);
            }
            if (!this.onlyConfig) {
                // Create tests location
                if (answers.testsLocation) {
                    this.createTestLocation(answers.testsLocation);
                }
                // Copy examples
                // For cucumber, only copy the cucumber examples.
                // For rest, copy all examples but cucumber.
                if (answers.runner === Runner.Cucumber) {
                    this.copyCucumberExamples(answers.examplesLocation || '');
                }
                else if (answers.addExamples) {
                    if ((0, constants_1.isWebTestingSetup)(answers)) {
                        this.copyExamples(answers.examplesLocation || '', answers.language === 'ts');
                    }
                    if ((0, constants_1.isAppTestingSetup)(answers)) {
                        yield this.copyAppTestingExamples(answers);
                    }
                    // For now the templates added only for JS
                    if (answers.language !== 'ts' && (0, constants_1.isWebTestingSetup)(answers)) {
                        this.copyTemplates(path_1.default.join(answers.examplesLocation || ''));
                    }
                }
            }
            // Setup component testing
            if ((_g = answers.testingType) === null || _g === void 0 ? void 0 : _g.includes('component')) {
                this.setupComponentTesting(answers);
            }
            // Generate configuration file
            const configDestPath = yield this.getConfigDestPath();
            this.generateConfig(answers, configDestPath);
            // Setup mobile
            const mobileHelperResult = {};
            if ((0, constants_1.isLocalMobileTestingSetup)(answers) && answers.mobilePlatform) {
                // answers.mobilePlatform will be undefined in case of empty or non-matching mobileBrowsers
                // hence, no need to setup any device.
                if (['android', 'both'].includes(answers.mobilePlatform)) {
                    logger_1.default.info('Running Android Setup...\n');
                    const androidSetup = new mobile_helper_1.AndroidSetup(Object.assign({ browsers: answers.mobileBrowsers || [] }, ((0, constants_1.isAppTestingSetup)(answers) && { appium: true })), this.rootDir);
                    mobileHelperResult.android = yield androidSetup.run();
                }
                if (['ios', 'both'].includes(answers.mobilePlatform)) {
                    logger_1.default.info('Running iOS Setup...\n');
                    const iosSetup = new mobile_helper_1.IosSetup({ mode: ['simulator', 'real'], setup: true });
                    mobileHelperResult.ios = yield iosSetup.run();
                }
            }
            if (!this.onlyConfig) {
                // Post instructions to run their first test
                this.postSetupInstructions(answers, mobileHelperResult);
            }
            else {
                // Post config instructions
                this.postConfigInstructions(answers);
            }
            if (answers.allowAnonymousMetrics) {
                this.pushAnonymousMetrics(answers);
            }
        });
    }
    askQuestions() {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const answers = Object.assign(Object.assign({ rootDir: this.rootDir, onlyConfig: this.onlyConfig, browsers: (_a = this.options) === null || _a === void 0 ? void 0 : _a.browser }, (((_b = this.options) === null || _b === void 0 ? void 0 : _b.mobile) && { mobile: true })), (((_c = this.options) === null || _c === void 0 ? void 0 : _c.native) && { native: true }));
            return yield (0, inquirer_1.prompt)(constants_1.QUESTIONAIRRE, answers);
        });
    }
    refineAnswers(answers) {
        var _a, _b;
        const onlyAppTestingSetup = answers.testingType && answers.testingType.length === 1 && answers.testingType[0] === 'app';
        const backendHasLocal = answers.backend && ['local', 'both'].includes(answers.backend);
        const backendHasRemote = answers.backend && ['remote', 'both'].includes(answers.backend);
        if (backendHasRemote) {
            answers.remoteName = 'remote';
            if (answers.cloudProvider !== 'other') {
                answers.remoteName = answers.cloudProvider;
            }
            answers.remoteEnv = {
                username: 'REMOTE_USERNAME',
                access_key: 'REMOTE_ACCESS_KEY'
            };
            if (answers.cloudProvider === 'browserstack') {
                answers.remoteEnv.username = 'BROWSERSTACK_USERNAME';
                answers.remoteEnv.access_key = 'BROWSERSTACK_ACCESS_KEY';
            }
            else if (answers.cloudProvider === 'saucelabs') {
                answers.remoteEnv.username = 'SAUCE_USERNAME';
                answers.remoteEnv.access_key = 'SAUCE_ACCESS_KEY';
            }
            if (!answers.remoteBrowsers) {
                if (answers.browsers) {
                    // we are testing on desktop browsers
                    // Copy all desktop browsers from `answers.browsers`.
                    answers.remoteBrowsers = [...answers.browsers];
                }
                else {
                    // we are not testing on desktop browsers
                    answers.remoteBrowsers = [];
                }
            }
            if (answers.mobile) {
                // we are testing on mobile browsers
                // right now, we are putting one config for all browsers.
                answers.mobileRemote = true;
            }
            // If backend is only remote (no local), delete answers.browsers (if present)
            // and set the defaultBrowser.
            if (!backendHasLocal) {
                if (answers.browsers) {
                    delete answers.browsers;
                }
                answers.defaultBrowser = answers.remoteBrowsers[0] || (onlyAppTestingSetup ? '' : 'chrome');
            }
        }
        if (backendHasLocal) {
            if (answers.browsers) {
                // we are testing on desktop browsers
                // Remove safari from answers.browsers for non-mac users
                if (process.platform !== 'darwin' && answers.browsers.includes('safari')) {
                    const pos = answers.browsers.indexOf('safari');
                    answers.browsers.splice(pos, 1);
                }
            }
            else {
                // we are not testing on desktop browsers
                answers.browsers = [];
            }
            if (answers.mobile) {
                // we are testing on mobile browsers
                if (answers.mobileBrowsers) {
                    // Remove safari from answers.mobileBrowsers for non-mac users (if present)
                    if (process.platform !== 'darwin' && answers.mobileBrowsers.includes('safari')) {
                        const pos = answers.mobileBrowsers.indexOf('safari');
                        answers.mobileBrowsers.splice(pos, 1);
                    }
                }
                else {
                    // copy browsers from `answers.browsers` (safari already removed)
                    // will be empty if `answers.browsers` is empty.
                    answers.mobileBrowsers = constants_1.MOBILE_BROWSER_CHOICES
                        .map((browserObj) => browserObj.value)
                        .filter((browser) => { var _a; return (_a = answers.browsers) === null || _a === void 0 ? void 0 : _a.includes(browser); });
                }
            }
            else {
                // we are not testing on mobile browsers
                answers.mobileBrowsers = [];
            }
            // Set defaultBrowser
            if (!answers.defaultBrowser) {
                answers.defaultBrowser = answers.browsers[0] || answers.mobileBrowsers[0] || (onlyAppTestingSetup ? '' : 'chrome');
            }
        }
        // Make sure baseUrl is not undefined
        answers.baseUrl = answers.baseUrl || '';
        // Always generate examples (for now)
        if (!this.onlyConfig) {
            answers.addExamples = true;
        }
        // Set testsLocation to default if not present
        if (!answers.testsLocation) {
            answers.testsLocation = defaults_json_1.default.testsLocation;
        }
        if (answers.addExamples && !answers.examplesLocation) {
            if (answers.runner === Runner.Cucumber) {
                answers.examplesLocation = path_1.default.join(answers.featurePath || '', DEFAULT_FOLDER);
            }
            else {
                answers.examplesLocation = DEFAULT_FOLDER;
            }
        }
        if (answers.mobile && !answers.mobilePlatform) {
            if ((_a = answers.mobileBrowsers) === null || _a === void 0 ? void 0 : _a.includes('safari')) {
                answers.mobilePlatform = 'ios';
            }
            if ((_b = answers.mobileBrowsers) === null || _b === void 0 ? void 0 : _b.some(browser => ['chrome', 'firefox'].includes(browser))) {
                if (answers.mobilePlatform === 'ios') {
                    answers.mobilePlatform = 'both';
                }
                else {
                    answers.mobilePlatform = 'android';
                }
            }
        }
        if ((0, constants_1.isAppTestingSetup)(answers) && !answers.mobilePlatform) {
            answers.mobilePlatform = 'android';
        }
        // Remove ios from mobilePlatform on non-mac systems (if present)
        if (process.platform !== 'darwin' && answers.mobilePlatform) {
            if (answers.mobilePlatform === 'both') {
                answers.mobilePlatform = 'android';
            }
            else if (answers.mobilePlatform === 'ios') {
                logger_1.default.error(`${ansi_colors_1.default.red('Error: Testing on iOS devices is not supported on non-mac systems.')}`);
                answers.mobilePlatform = undefined;
            }
        }
        if (answers.uiFramework) {
            answers.plugins = answers.plugins || [];
            answers.plugins.push(`@nightwatch/${answers.uiFramework}`);
        }
    }
    identifyPackagesToInstall(answers) {
        const packages = ['nightwatch'];
        if (answers.language === 'ts') {
            packages.push('typescript', '@types/nightwatch', 'ts-node');
        }
        if (answers.runner === Runner.Cucumber) {
            packages.push('@cucumber/cucumber');
        }
        if (answers.seleniumServer) {
            packages.push('@nightwatch/selenium-server');
        }
        if ((0, constants_1.isAppTestingSetup)(answers) && answers.backend !== 'remote') {
            packages.push('appium');
        }
        if (answers.plugins) {
            packages.push(...answers.plugins);
        }
        // Identify packages already installed and don't install them again
        const packageJson = JSON.parse(node_fs_1.default.readFileSync(path_1.default.join(this.rootDir, 'package.json'), 'utf-8'));
        const packagesToInstall = packages.filter((pack) => {
            var _a, _b;
            // eslint-disable-next-line
            return !((_a = packageJson.devDependencies) === null || _a === void 0 ? void 0 : _a.hasOwnProperty(pack)) && !((_b = packageJson.dependencies) === null || _b === void 0 ? void 0 : _b.hasOwnProperty(pack));
        });
        // Packages to always upgrade
        if ((0, constants_1.isLocalMobileTestingSetup)(answers)) {
            packagesToInstall.push('@nightwatch/mobile-helper');
        }
        return packagesToInstall;
    }
    installPackages(packagesToInstall) {
        if (packagesToInstall.length === 0) {
            return;
        }
        logger_1.default.info('Installing the following packages:');
        for (const pack of packagesToInstall) {
            logger_1.default.info(`- ${pack}`);
        }
        logger_1.default.info();
        for (const pack of packagesToInstall) {
            logger_1.default.info(`Installing ${ansi_colors_1.default.green(pack)}`);
            try {
                (0, child_process_1.execSync)(`npm install ${pack} --save-dev`, {
                    stdio: ['inherit', 'pipe', 'inherit'],
                    cwd: this.rootDir
                });
                logger_1.default.info(ansi_colors_1.default.green('Done!'), '\n');
            }
            catch (err) {
                logger_1.default.error(`Failed to install ${pack}. Please run 'npm install ${pack} --save-dev' later.\n`);
            }
        }
    }
    setupTypescript() {
        const tsConfigPath = path_1.default.join(this.rootDir, 'tsconfig.json');
        // Generate a new tsconfig.json file if not already present.
        if (!node_fs_1.default.existsSync(tsConfigPath)) {
            (0, child_process_1.execSync)('npx tsc --init', {
                stdio: 'inherit',
                cwd: this.rootDir
            });
            logger_1.default.info();
        }
        // Generate a new tsconfig.json file to be used by ts-node, if not already present.
        const tsConfigNightwatchPath1 = path_1.default.join(this.rootDir, 'nightwatch', 'tsconfig.json');
        const tsConfigNightwatchPath2 = path_1.default.join(this.rootDir, 'tsconfig.nightwatch.json');
        if (!node_fs_1.default.existsSync(tsConfigNightwatchPath1) && !node_fs_1.default.existsSync(tsConfigNightwatchPath2)) {
            const tsConfigSrcPath = path_1.default.join(__dirname, '..', 'assets', 'tsconfig.json');
            const tsConfigDestPath = path_1.default.join(this.rootDir, 'nightwatch', 'tsconfig.json');
            try {
                node_fs_1.default.mkdirSync(path_1.default.join(this.rootDir, 'nightwatch'));
                // eslint-disable-next-line
            }
            catch (err) { }
            node_fs_1.default.copyFileSync(tsConfigSrcPath, tsConfigDestPath);
        }
        // Set outDir property to null for now.
        this.otherInfo.tsOutDir = '';
    }
    setupComponentTesting(answers) {
        if (answers.uiFramework === 'react') {
            const componentConfigPath = path_1.default.join(__dirname, '..', 'assets', 'component-config');
            const nightwatchPath = path_1.default.join(this.rootDir, DEFAULT_FOLDER);
            try {
                node_fs_1.default.mkdirSync(nightwatchPath);
                // eslint-disable-next-line
            }
            catch (err) { }
            // Generate a new index.jsx file
            const reactIndexSrcPath = path_1.default.join(componentConfigPath, 'index.jsx');
            const reactIndexDestPath = path_1.default.join(nightwatchPath, 'index.jsx');
            node_fs_1.default.copyFileSync(reactIndexSrcPath, reactIndexDestPath);
        }
    }
    checkJavaInstallation() {
        try {
            (0, child_process_1.execSync)('java -version', {
                stdio: 'pipe',
                cwd: this.rootDir
            });
        }
        catch (err) {
            this.otherInfo.javaNotInstalled = true;
        }
    }
    getConfigDestPath() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if ((_a = this.options) === null || _a === void 0 ? void 0 : _a.yes) {
                logger_1.default.info('Auto-generating a configuration file...\n');
            }
            else {
                logger_1.default.info('Generating a configuration file based on your responses...\n');
            }
            // check for ESM project
            const packageJson = JSON.parse(node_fs_1.default.readFileSync(path_1.default.join(this.rootDir, 'package.json'), 'utf-8'));
            const usingESM = packageJson.type === 'module';
            this.otherInfo.usingESM = usingESM;
            const configExt = usingESM ? '.conf.cjs' : '.conf.js';
            const configDestPath = path_1.default.join(this.rootDir, `nightwatch${configExt}`);
            if (node_fs_1.default.existsSync(configDestPath)) {
                logger_1.default.info(ansi_colors_1.default.yellow(`There seems to be another config file located at "${configDestPath}".\n`));
                const answers = yield (0, inquirer_1.prompt)(constants_1.CONFIG_DEST_QUES, { rootDir: this.rootDir, configExt });
                // Adding a newline after questions.
                logger_1.default.info();
                if (!answers.overwrite) {
                    const configFileName = `${answers.newFileName}${configExt}`;
                    this.otherInfo.nonDefaultConfigName = configFileName;
                    return path_1.default.join(this.rootDir, configFileName);
                }
            }
            return configDestPath;
        });
    }
    generateConfig(answers, configDestPath) {
        const templateFile = path_1.default.join(__dirname, '..', 'src', 'config', 'main.ejs');
        const src_folders = []; // to go into the config file as the value of src_folders property.
        const page_objects_path = []; // to go as the value of page_objects_configs property.
        const custom_commands_path = []; // to go as the value of custom_commands_path property.
        const custom_assertions_path = []; // to go as the value of custom_assertions_path property.
        const feature_path = answers.featurePath || ''; // to be used in cucumber feature_path property.
        const plugins = answers.plugins || []; // to go as the value of plugins property.
        const testsJsSrc = path_1.default.join(this.otherInfo.tsOutDir || '', answers.testsLocation || '');
        if (testsJsSrc !== '.') {
            if (answers.testsLocation === answers.examplesLocation && answers.language === 'js' && answers.runner !== Runner.Cucumber) {
                // examples are being put as a boilerplate in testsLocation with main tests in
                // EXAMPLE_TEST_FOLDER sub-directory (only done for JS-Nightwatch and JS-Mocha).
                src_folders.push(path_1.default.join(testsJsSrc, EXAMPLE_TEST_FOLDER));
            }
            else {
                src_folders.push(testsJsSrc);
            }
            this.otherInfo.testsJsSrc = testsJsSrc;
        }
        if (answers.addExamples && answers.runner !== Runner.Cucumber) {
            // Add examplesLocation to src_folders, if different from testsLocation.
            // Don't add for cucumber examples (for now, as addition of examples depends upon featurePath in copyCucumberExamples).
            const examplesJsSrc = path_1.default.join(this.otherInfo.tsOutDir || '', answers.examplesLocation || '');
            if (examplesJsSrc !== testsJsSrc) {
                if (answers.language === 'js') {
                    // Only for JS-Nightwatch and JS-Mocha.
                    src_folders.push(path_1.default.join(examplesJsSrc, EXAMPLE_TEST_FOLDER));
                }
                else {
                    src_folders.push(examplesJsSrc);
                }
            }
            this.otherInfo.examplesJsSrc = examplesJsSrc;
            if ((0, constants_1.isWebTestingSetup)(answers) && answers.language === 'js') {
                // Right now, we only ship page-objects/custom-commands/custom-assertions
                // examples for web-tests in JS (Nightwatch and Mocha test runner) only.
                page_objects_path.push(`${path_1.default.join(examplesJsSrc, 'page-objects')}`);
                custom_commands_path.push(`${path_1.default.join(examplesJsSrc, 'custom-commands')}`);
                custom_assertions_path.push(`${path_1.default.join(examplesJsSrc, 'custom-assertions')}`);
            }
        }
        const tplData = node_fs_1.default.readFileSync(templateFile).toString();
        let rendered = ejs_1.default.render(tplData, {
            plugins: JSON.stringify(plugins).replace(/"/g, '\'').replace(/\\\\/g, '/'),
            src_folders: JSON.stringify(src_folders).replace(/"/g, '\'').replace(/\\\\/g, '/'),
            page_objects_path: JSON.stringify(page_objects_path).replace(/"/g, '\'').replace(/\\\\/g, '/'),
            custom_commands_path: JSON.stringify(custom_commands_path).replace(/"/g, '\'').replace(/\\\\/g, '/'),
            custom_assertions_path: JSON.stringify(custom_assertions_path).replace(/"/g, '\'').replace(/\\\\/g, '/'),
            feature_path: feature_path.replace(/\\/g, '/'),
            client_id: this.client_id,
            dotExe: process.platform === 'win32' ? '.exe' : '',
            answers
        });
        rendered = (0, utils_1.stripControlChars)(rendered);
        try {
            node_fs_1.default.writeFileSync(configDestPath, rendered, { encoding: 'utf-8' });
            logger_1.default.info(`${ansi_colors_1.default.green((0, utils_1.symbols)().ok + ' Success!')} Configuration file generated at: "${configDestPath}".`);
            if (this.otherInfo.nonDefaultConfigName) {
                logger_1.default.info(`To use this configuration file, run the tests using ${ansi_colors_1.default.magenta('--config')} flag.`);
            }
            // Add a newline
            logger_1.default.info();
            return true;
        }
        catch (err) {
            logger_1.default.error('Failed to generate Nightwatch config.');
            logger_1.default.error('Please run the init command again, or a config file will be auto-generated when you run your first test.');
            return false;
        }
    }
    identifyWebdriversToInstall(answers) {
        var _a, _b, _c, _d, _e, _f;
        const webdrivers = [];
        const localWebTestingOnFirefox = ((_a = answers.browsers) === null || _a === void 0 ? void 0 : _a.includes('firefox')) || ((_b = answers.mobileBrowsers) === null || _b === void 0 ? void 0 : _b.includes('firefox'));
        if (localWebTestingOnFirefox) {
            webdrivers.push('geckodriver');
        }
        const localWebTestingOnChrome = ((_c = answers.browsers) === null || _c === void 0 ? void 0 : _c.includes('chrome')) || ((_d = answers.mobileBrowsers) === null || _d === void 0 ? void 0 : _d.includes('chrome'));
        const localAppTestingOnAndroid = ((0, constants_1.isAppTestingSetup)(answers) && answers.backend !== 'remote' &&
            answers.mobilePlatform && ['android', 'both'].includes(answers.mobilePlatform));
        if (localWebTestingOnChrome || localAppTestingOnAndroid) {
            webdrivers.push('chromedriver');
        }
        const localWebTestingOnSafari = ((_e = answers.browsers) === null || _e === void 0 ? void 0 : _e.includes('safari')) || ((_f = answers.mobileBrowsers) === null || _f === void 0 ? void 0 : _f.includes('safari'));
        const localAppTestingOnIos = ((0, constants_1.isAppTestingSetup)(answers) && answers.backend !== 'remote' &&
            answers.mobilePlatform && ['ios', 'both'].includes(answers.mobilePlatform));
        if (localWebTestingOnSafari || localAppTestingOnIos) {
            webdrivers.push('safaridriver');
        }
        return webdrivers;
    }
    installWebdrivers(webdriversToInstall) {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.default.info('Installing/updating the following webdrivers:');
            for (const webdriver of webdriversToInstall) {
                logger_1.default.info(`- ${webdriver}`);
            }
            logger_1.default.info();
            const driversDownloadedFromNPM = {
                geckodriver: 'Firefox',
                chromedriver: 'Chrome'
            };
            for (const webdriver of webdriversToInstall) {
                if (webdriver in driversDownloadedFromNPM) {
                    logger_1.default.info(`Installing webdriver for ${driversDownloadedFromNPM[webdriver]} (${webdriver})...`);
                    try {
                        (0, child_process_1.execSync)(`npm install ${webdriver} --save-dev`, {
                            stdio: ['inherit', 'pipe', 'inherit'],
                            cwd: this.rootDir
                        });
                        logger_1.default.info(ansi_colors_1.default.green('Done!'), '\n');
                    }
                    catch (err) {
                        logger_1.default.error(`Failed to install ${webdriver}. Please run 'npm install ${webdriver} --save-dev' later.\n`);
                    }
                }
            }
            if (webdriversToInstall.includes('safaridriver')) {
                try {
                    const answers = yield (0, inquirer_1.prompt)([
                        {
                            type: 'list',
                            name: 'safaridriver',
                            message: 'Enable safaridriver (requires sudo password, skip if already enabled)?',
                            choices: [
                                { name: 'Yes', value: true },
                                { name: 'No, skip for now', value: false }
                            ],
                            default: 1
                        }
                    ]);
                    if (answers.safaridriver) {
                        logger_1.default.info();
                        logger_1.default.info('Enabling safaridriver...');
                        (0, child_process_1.execSync)('sudo safaridriver --enable', {
                            stdio: ['inherit', 'pipe', 'inherit'],
                            cwd: this.rootDir
                        });
                        logger_1.default.info(ansi_colors_1.default.green('Done!'), '\n');
                    }
                    else {
                        logger_1.default.info('Please run \'sudo safaridriver --enable\' command to enable safaridriver later.\n');
                    }
                }
                catch (err) {
                    logger_1.default.error('Failed to enable safaridriver. Please run \'sudo safaridriver --enable\' later.\n');
                }
            }
        });
    }
    createTestLocation(testsLocation) {
        try {
            node_fs_1.default.mkdirSync(path_1.default.join(this.rootDir, testsLocation), { recursive: true });
            // eslint-disable-next-line
        }
        catch (err) { }
    }
    copyCucumberExamples(examplesLocation) {
        // If the featurePath (part of examplesLocation) contains **, no way of knowing where to put
        // example feature files (maybe in the most outside folder by creating a new example dir?)
        // Skipping all paths with '*' for now.
        if (examplesLocation.includes('*')) {
            return;
        }
        logger_1.default.info('Generating example for CucumberJS...');
        this.otherInfo.cucumberExamplesAdded = true;
        const exampleDestPath = path_1.default.join(this.rootDir, examplesLocation);
        if (node_fs_1.default.existsSync(exampleDestPath)) {
            logger_1.default.info(`Example already exists at '${examplesLocation}'. Skipping...`, '\n');
            return;
        }
        node_fs_1.default.mkdirSync(exampleDestPath, { recursive: true });
        const nightwatchModulePath = path_1.default.dirname(require.resolve('nightwatch/package.json', { paths: [this.rootDir] }));
        const exampleSrcPath = path_1.default.join(nightwatchModulePath, 'examples', 'cucumber-js', 'features');
        (0, utils_1.copy)(exampleSrcPath, exampleDestPath);
        logger_1.default.info(`${ansi_colors_1.default.green((0, utils_1.symbols)().ok + ' Success!')} Generated an example for CucumberJS at "${examplesLocation}".\n`);
    }
    copyExamples(examplesLocation, typescript) {
        logger_1.default.info('Generating example files...');
        const examplesDestPath = path_1.default.join(this.rootDir, examplesLocation); // this is different from this.otherInfo.examplesJsSrc
        try {
            node_fs_1.default.mkdirSync(examplesDestPath, { recursive: true });
            // eslint-disable-next-line
        }
        catch (err) { }
        const examplesDestFiles = node_fs_1.default.readdirSync(examplesDestPath);
        if ((typescript && examplesDestFiles.length > 1) || (!typescript && examplesDestFiles.length > 0)) {
            logger_1.default.info(`Examples already exists at '${examplesLocation}'. Skipping...`, '\n');
            return;
        }
        let examplesSrcPath;
        if (typescript) {
            examplesSrcPath = path_1.default.join(__dirname, '..', 'assets', 'ts-examples');
        }
        else {
            examplesSrcPath = path_1.default.join(__dirname, '..', 'assets', 'js-examples-new');
        }
        (0, utils_1.copy)(examplesSrcPath, examplesDestPath);
        logger_1.default.info(`${ansi_colors_1.default.green((0, utils_1.symbols)().ok + ' Success!')} Generated some example files at '${examplesLocation}'.\n`);
    }
    copyAppTestingExamples(answers) {
        return __awaiter(this, void 0, void 0, function* () {
            const examplesLocation = answers.examplesLocation || '';
            const lang = answers.language || 'js';
            const mobilePlatforms = [];
            if (answers.mobilePlatform) {
                if (answers.mobilePlatform === 'both') {
                    mobilePlatforms.push('android', 'ios');
                }
                else {
                    mobilePlatforms.push(answers.mobilePlatform);
                }
            }
            logger_1.default.info('Generating mobile-app example tests...\n');
            const examplesDestPath = path_1.default.join(this.rootDir, examplesLocation, lang === 'js' ? EXAMPLE_TEST_FOLDER : '', 'mobile-app-tests');
            const appDestPath = path_1.default.join(this.rootDir, examplesLocation, 'sample-apps');
            try {
                node_fs_1.default.mkdirSync(examplesDestPath, { recursive: true });
                node_fs_1.default.mkdirSync(appDestPath, { recursive: true });
                // eslint-disable-next-line
            }
            catch (err) { }
            for (const platform of mobilePlatforms) {
                const examplesSrcPath = path_1.default.join(__dirname, '..', 'assets', 'mobile-app-tests', `${platform}-${lang}`);
                (0, utils_1.copy)(examplesSrcPath, examplesDestPath);
                logger_1.default.info(`Downloading sample ${platform} app...`);
                const downloadUrl = downloads_json_1.default.wikipedia[platform];
                const downloaded = yield (0, utils_1.downloadWithProgressBar)(downloadUrl, appDestPath);
                if (!downloaded) {
                    logger_1.default.info(`${ansi_colors_1.default.red('Download Failed!')} You can download it from ${downloadUrl} and save it to '${path_1.default.join(examplesLocation, 'sample-apps')}' inside your project root dir.\n`);
                }
            }
        });
    }
    copyTemplates(examplesLocation) {
        logger_1.default.info('Generating template files...');
        // Set templatesGenerated to true even if skipped, since in that case
        // templates are already present.
        this.otherInfo.templatesGenerated = true;
        const templatesLocation = path_1.default.join(examplesLocation, 'templates');
        const templatesDestPath = path_1.default.join(this.rootDir, templatesLocation);
        try {
            node_fs_1.default.mkdirSync(templatesDestPath, { recursive: true });
            // eslint-disable-next-line
        }
        catch (err) { }
        if (node_fs_1.default.readdirSync(templatesDestPath).length) {
            logger_1.default.info(`Templates already exists at '${templatesLocation}'. Skipping...`, '\n');
            return;
        }
        const templatesSrcPath = path_1.default.join(__dirname, '..', 'assets', 'templates');
        (0, utils_1.copy)(templatesSrcPath, templatesDestPath);
        logger_1.default.info(`${ansi_colors_1.default.green((0, utils_1.symbols)().ok + ' Success!')} Generated some templates files at '${templatesLocation}'.\n`);
    }
    postSetupInstructions(answers, mobileHelperResult) {
        var _a, _b, _c, _d, _e, _f;
        // Instructions for setting host, port, username and password for remote.
        if (answers.backend && ['remote', 'both'].includes(answers.backend)) {
            logger_1.default.info(ansi_colors_1.default.red('IMPORTANT'));
            if (answers.cloudProvider === 'other') {
                let configFileName = this.otherInfo.usingESM ? 'nightwatch.conf.cjs' : 'nightwatch.conf.js';
                if (this.otherInfo.nonDefaultConfigName) {
                    configFileName = this.otherInfo.nonDefaultConfigName;
                }
                logger_1.default.info(`To run tests on your remote device, please set the ${ansi_colors_1.default.magenta('host')} and ${ansi_colors_1.default.magenta('port')} property in your ${configFileName} file.`);
                logger_1.default.info('These can be located at:');
                logger_1.default.info(`{\n  ...\n  "test_settings": {\n    ...\n    "${answers.remoteName}": {\n      "selenium": {\n        ${ansi_colors_1.default.cyan('"host":')}\n        ${ansi_colors_1.default.cyan('"port":')}\n      }\n    }\n  }\n}`, '\n');
                logger_1.default.info('Please set the credentials (if any) required to run tests on your cloud provider or remote selenium-server, by setting the below env variables:');
            }
            else {
                logger_1.default.info('Please set the credentials required to run tests on your cloud provider, by setting the below env variables:');
            }
            logger_1.default.info(`- ${ansi_colors_1.default.cyan((_a = answers.remoteEnv) === null || _a === void 0 ? void 0 : _a.username)}`);
            logger_1.default.info(`- ${ansi_colors_1.default.cyan((_b = answers.remoteEnv) === null || _b === void 0 ? void 0 : _b.access_key)}`);
            logger_1.default.info('(.env files are also supported)', '\n');
        }
        logger_1.default.info();
        const relativeToRootDir = path_1.default.relative(process.cwd(), this.rootDir) || '.';
        // For now the templates added only for JS
        if (this.otherInfo.templatesGenerated) {
            logger_1.default.info(ansi_colors_1.default.green('📃 TEMPLATE TESTS'), '\n');
            logger_1.default.info('To get started, checkout the following templates. Skip/delete them if you are an experienced user.');
            logger_1.default.info(ansi_colors_1.default.cyan(`  1. Title Assertion (${path_1.default.join(relativeToRootDir, answers.examplesLocation || '', 'templates', 'titleAssertion.js')})`));
            logger_1.default.info(ansi_colors_1.default.cyan(`  2. Login (${path_1.default.join(relativeToRootDir, answers.examplesLocation || '', 'templates', 'login.js')})`));
            logger_1.default.info();
        }
        logger_1.default.info(ansi_colors_1.default.green('✨ SETUP COMPLETE'));
        (0, child_process_1.execSync)('npx nightwatch --version', {
            stdio: 'inherit',
            cwd: this.rootDir
        });
        // Join Discord and GitHub
        logger_1.default.info('💬 Join our Discord community to find answers to your issues or queries. Or just join and say hi.');
        logger_1.default.info(ansi_colors_1.default.cyan('   https://discord.gg/SN8Da2X'), '\n');
        let exampleCommandsShared = false;
        let configFlag = '';
        if (this.otherInfo.nonDefaultConfigName) {
            configFlag = ` --config ${this.otherInfo.nonDefaultConfigName}`;
        }
        if ((0, constants_1.isWebTestingSetup)(answers) && !((_c = this.options) === null || _c === void 0 ? void 0 : _c.mobile)) {
            // web-testing setup and --mobile flag is not used (testing on desktop browsers).
            exampleCommandsShared = true;
            logger_1.default.info(ansi_colors_1.default.green('🚀 RUN EXAMPLE TESTS'), '\n');
            if (this.rootDir !== process.cwd()) {
                logger_1.default.info('First, change directory to the root dir of your project:');
                logger_1.default.info(ansi_colors_1.default.cyan(`  cd ${relativeToRootDir}`), '\n');
            }
            let envFlag = '';
            if (answers.backend === 'remote') {
                envFlag = ` --env ${answers.remoteName}.${answers.defaultBrowser}`;
            }
            if (answers.runner === Runner.Cucumber) {
                logger_1.default.info('To run your tests with CucumberJS, simply run:');
                logger_1.default.info(ansi_colors_1.default.cyan(`  npx nightwatch${envFlag}${configFlag}`), '\n');
                if (this.otherInfo.cucumberExamplesAdded) {
                    logger_1.default.info('To run an example test with CucumberJS, run:');
                    logger_1.default.info(ansi_colors_1.default.cyan(`  npx nightwatch ${answers.examplesLocation}${envFlag}${configFlag}`), '\n');
                }
                logger_1.default.info('For more details on using CucumberJS with Nightwatch, visit:');
                logger_1.default.info(ansi_colors_1.default.cyan('  https://nightwatchjs.org/guide/third-party-runners/cucumberjs-nightwatch-integration.html'));
            }
            else if (answers.addExamples) {
                if (answers.language === 'ts') {
                    logger_1.default.info('To run all examples, run:');
                    logger_1.default.info(ansi_colors_1.default.cyan(`  npx nightwatch .${path_1.default.sep}${this.otherInfo.examplesJsSrc}${envFlag}${configFlag}\n`));
                    logger_1.default.info('To run a single example (github.ts), run:');
                    logger_1.default.info(ansi_colors_1.default.cyan(`  npx nightwatch .${path_1.default.sep}${path_1.default.join(this.otherInfo.examplesJsSrc || '', 'github.ts')}${envFlag}${configFlag}\n`));
                }
                else {
                    logger_1.default.info('To run all examples, run:');
                    logger_1.default.info(ansi_colors_1.default.cyan(`  npx nightwatch .${path_1.default.sep}${path_1.default.join(this.otherInfo.examplesJsSrc || '', EXAMPLE_TEST_FOLDER)}${envFlag}${configFlag}\n`));
                    logger_1.default.info('To run a single example (ecosia.js), run:');
                    logger_1.default.info(ansi_colors_1.default.cyan(`  npx nightwatch .${path_1.default.sep}${path_1.default.join(this.otherInfo.examplesJsSrc || '', EXAMPLE_TEST_FOLDER, 'basic', 'ecosia.js')}${envFlag}${configFlag}\n`));
                }
            }
            else {
                logger_1.default.info(`A few examples are available at '${path_1.default.join('node_modules', 'nightwatch', 'examples')}'.\n`);
                logger_1.default.info('To run a single example (ecosia.js), try:');
                logger_1.default.info(ansi_colors_1.default.cyan(`  npx nightwatch ${path_1.default.join('node_modules', 'nightwatch', 'examples', 'tests', 'ecosia.js')}${envFlag}${configFlag}`), '\n');
                logger_1.default.info('To run all examples, try:');
                logger_1.default.info(ansi_colors_1.default.cyan(`  npx nightwatch ${path_1.default.join('node_modules', 'nightwatch', 'examples')}${envFlag}${configFlag}`), '\n');
            }
        }
        if (answers.seleniumServer) {
            logger_1.default.info('[Selenium Server]\n');
            if (this.otherInfo.javaNotInstalled) {
                logger_1.default.info('Java Development Kit (minimum v7) is required to run selenium-server locally. Download from here:');
                logger_1.default.info(ansi_colors_1.default.cyan('  https://www.oracle.com/technetwork/java/javase/downloads/index.html'), '\n');
            }
            logger_1.default.info('To run tests on your local selenium-server, use command:');
            logger_1.default.info(ansi_colors_1.default.cyan(`  npx nightwatch --env selenium_server${configFlag}`), '\n');
        }
        if ((_d = answers.browsers) === null || _d === void 0 ? void 0 : _d.includes('edge')) {
            logger_1.default.info(`${ansi_colors_1.default.red('Note:')} Microsoft Edge Webdriver is not installed automatically.`);
            logger_1.default.info('Please follow the below link ("Download" and "Standalone Usage" sections) to setup EdgeDriver manually:');
            logger_1.default.info(ansi_colors_1.default.cyan('  https://nightwatchjs.org/guide/browser-drivers-setup/edgedriver.html'), '\n');
        }
        // MOBILE WEB AND APP TESTS
        const cucumberExample = `npx nightwatch${configFlag}`;
        const mobileTsExample = `npx nightwatch .${path_1.default.sep}${path_1.default.join(this.otherInfo.examplesJsSrc || '', 'github.ts')}${configFlag}`;
        const mobileJsExample = `npx nightwatch .${path_1.default.sep}${path_1.default.join(this.otherInfo.examplesJsSrc || '', EXAMPLE_TEST_FOLDER, 'basic', 'ecosia.js')}${configFlag}`;
        const mobileExampleCommand = (envFlag) => {
            if (answers.runner === Runner.Cucumber) {
                return `${cucumberExample}${envFlag}`;
            }
            if (answers.language === 'ts') {
                return `${mobileTsExample}${envFlag}`;
            }
            return `${mobileJsExample}${envFlag}`;
        };
        const appTsExample = (mobilePlatform) => {
            return `npx nightwatch .${path_1.default.sep}${path_1.default.join(this.otherInfo.examplesJsSrc || '', 'mobile-app-tests', `wikipedia-${mobilePlatform}.ts`)}${configFlag}`;
        };
        const appJsExample = (mobilePlatform) => {
            return `npx nightwatch .${path_1.default.sep}${path_1.default.join(this.otherInfo.examplesJsSrc || '', EXAMPLE_TEST_FOLDER, 'mobile-app-tests', `wikipedia-${mobilePlatform}.js`)}${configFlag}`;
        };
        const appExampleCommand = (envFlag, mobilePlatform) => {
            // no cucumber app-tests for now
            if (answers.language === 'ts') {
                return `${appTsExample(mobilePlatform)}${envFlag}`;
            }
            return `${appJsExample(mobilePlatform)}${envFlag}`;
        };
        const cucumberAppTestingOnly = answers.runner === Runner.Cucumber && (0, constants_1.isAppTestingSetup)(answers) && !answers.mobile;
        if ((0, constants_1.isLocalMobileTestingSetup)(answers) && answers.mobilePlatform && !cucumberAppTestingOnly) {
            exampleCommandsShared = true;
            logger_1.default.info(ansi_colors_1.default.green('🚀 RUN MOBILE EXAMPLE TESTS'), '\n');
            if (['android', 'both'].includes(answers.mobilePlatform)) {
                const errorHelp = 'Please go through the setup logs above to know the actual cause of failure.\n\nOr, re-run the following commands:';
                const appiumFlag = (0, constants_1.isAppTestingSetup)(answers) ? ' --appium' : '';
                const setupMsg = `  To setup Android, run: ${ansi_colors_1.default.gray.italic('npx @nightwatch/mobile-helper android' + appiumFlag)}\n` +
                    `  For Android help, run: ${ansi_colors_1.default.gray.italic('npx @nightwatch/mobile-helper android --help')}`;
                const browsers = ((_e = answers.mobileBrowsers) === null || _e === void 0 ? void 0 : _e.filter((browser) => ['chrome', 'firefox'].includes(browser))) || [];
                const realAndroidTestCommand = (newline = '') => {
                    const commands = [];
                    commands.push('▶ To run an example test on Real Android device');
                    commands.push('  * Make sure your device is connected with USB Debugging turned on.');
                    commands.push('  * Make sure required browsers are installed.');
                    if (answers.mobile && browsers.length) {
                        commands.push('  For mobile web tests, run:');
                        for (const browser of browsers) {
                            const envFlag = ` --env android.real.${browser}`;
                            commands.push(`    ${ansi_colors_1.default.cyan(mobileExampleCommand(envFlag))}${newline}`);
                        }
                    }
                    if ((0, constants_1.isAppTestingSetup)(answers) && answers.runner !== Runner.Cucumber) {
                        commands.push('  For mobile app tests, run:');
                        const envFlag = ' --env app.android.real';
                        commands.push(`    ${ansi_colors_1.default.cyan(appExampleCommand(envFlag, 'android'))}${newline}`);
                    }
                    return commands.join('\n');
                };
                const emulatorAndroidTestCommand = (newline = '') => {
                    const commands = [];
                    commands.push('▶ To run an example test on Android Emulator');
                    if (answers.mobile && browsers.length) {
                        commands.push('  For mobile web tests, run:');
                        for (const browser of browsers) {
                            const envFlag = ` --env android.emulator.${browser}`;
                            commands.push(`    ${ansi_colors_1.default.cyan(mobileExampleCommand(envFlag))}${newline}`);
                        }
                    }
                    if ((0, constants_1.isAppTestingSetup)(answers) && answers.runner !== Runner.Cucumber) {
                        commands.push('  For mobile app tests, run:');
                        const envFlag = ' --env app.android.emulator';
                        commands.push(`    ${ansi_colors_1.default.cyan(appExampleCommand(envFlag, 'android'))}${newline}`);
                    }
                    return commands.join('\n');
                };
                const testCommands = `Once setup is complete...\n\n${realAndroidTestCommand()}\n\n${emulatorAndroidTestCommand()}`;
                if (!mobileHelperResult.android) {
                    // mobileHelperResult.android is undefined or false
                    logger_1.default.error((0, boxen_1.default)(`${ansi_colors_1.default.red('Android setup failed...')}\n\n${errorHelp}\n${setupMsg}\n\n${testCommands}`, { padding: 1 }));
                }
                else if (mobileHelperResult.android === true) {
                    // do nothing (command passed but verification/setup not initiated)
                    // true is returned in cases of --help command.
                }
                else if (mobileHelperResult.android.status === false) {
                    if (mobileHelperResult.android.setup) {
                        logger_1.default.error((0, boxen_1.default)(`${ansi_colors_1.default.red('Android setup failed...')}\n\n${errorHelp}\n${setupMsg}\n\n${testCommands}`, { padding: 1 }));
                    }
                    else {
                        logger_1.default.info((0, boxen_1.default)(`${ansi_colors_1.default.red('Android setup skipped...')}\n\n${setupMsg}\n\n${testCommands}`, { padding: 1 }));
                    }
                }
                else {
                    // mobileHelperResult.android.status is true.
                    if (this.rootDir !== process.cwd()) {
                        logger_1.default.info('First, change directory to the root dir of your project:');
                        logger_1.default.info(ansi_colors_1.default.cyan(`  cd ${relativeToRootDir}`), '\n');
                    }
                    if (['real', 'both'].includes(mobileHelperResult.android.mode)) {
                        logger_1.default.info(realAndroidTestCommand(), '\n');
                    }
                    if (['emulator', 'both'].includes(mobileHelperResult.android.mode)) {
                        logger_1.default.info(emulatorAndroidTestCommand(), '\n');
                    }
                }
            }
            if (['ios', 'both'].includes(answers.mobilePlatform)) {
                const setupHelp = 'Please follow the guide above to complete the setup.\n\nOr, re-run the following commands:';
                const setupCommand = `  For iOS setup, run: ${ansi_colors_1.default.gray.italic('npx @nightwatch/mobile-helper ios --setup')}\n` +
                    `  For iOS help, run: ${ansi_colors_1.default.gray.italic('npx @nightwatch/mobile-helper ios --help')}`;
                const safariBrowserPresent = (_f = answers.mobileBrowsers) === null || _f === void 0 ? void 0 : _f.includes('safari');
                const realIosTestCommand = () => {
                    const commands = [];
                    commands.push('▶ To run an example test on real iOS device');
                    if (answers.mobile && safariBrowserPresent) {
                        commands.push('  For mobile web tests, run:');
                        commands.push(`    ${ansi_colors_1.default.cyan(mobileExampleCommand(' --env ios.real.safari'))}`);
                    }
                    if ((0, constants_1.isAppTestingSetup)(answers) && answers.runner !== Runner.Cucumber) {
                        commands.push('  For mobile app tests, run:');
                        commands.push(`    ${ansi_colors_1.default.cyan(appExampleCommand(' --env app.ios.real', 'ios'))}`);
                    }
                    return commands.join('\n');
                };
                const simulatorIosTestCommand = () => {
                    const commands = [];
                    commands.push('▶ To run an example test on iOS simulator');
                    if (answers.mobile && safariBrowserPresent) {
                        commands.push('  For mobile web tests, run:');
                        commands.push(`    ${ansi_colors_1.default.cyan(mobileExampleCommand(' --env ios.simulator.safari'))}`);
                    }
                    if ((0, constants_1.isAppTestingSetup)(answers) && answers.runner !== Runner.Cucumber) {
                        commands.push('  For mobile app tests, run:');
                        commands.push(`    ${ansi_colors_1.default.cyan(appExampleCommand(' --env app.ios.simulator', 'ios'))}`);
                    }
                    return commands.join('\n');
                };
                const testCommand = `After completing the setup...\n\n${realIosTestCommand()}\n\n${simulatorIosTestCommand()}`;
                if (!mobileHelperResult.ios) {
                    logger_1.default.error((0, boxen_1.default)(`${ansi_colors_1.default.red('iOS setup failed...')}\n\n${setupCommand}\n\n${testCommand}`, { padding: 1 }));
                }
                else if (typeof mobileHelperResult.ios === 'object') {
                    if (mobileHelperResult.ios.real) {
                        logger_1.default.info(realIosTestCommand(), '\n');
                    }
                    if (mobileHelperResult.ios.simulator) {
                        logger_1.default.info(simulatorIosTestCommand(), '\n');
                    }
                    if (!mobileHelperResult.ios.real || !mobileHelperResult.ios.simulator) {
                        logger_1.default.error((0, boxen_1.default)(`${ansi_colors_1.default.yellow('iOS setup incomplete...')}\n\n${setupHelp}\n${setupCommand}\n\n${testCommand}`, { padding: 1 }));
                    }
                }
            }
        }
        if (!exampleCommandsShared && (0, constants_1.isRemoteMobileTestingSetup)(answers) && answers.cloudProvider === 'browserstack') {
            // no other test run commands are printed and remote with mobile (web/app) is selected.
            // TODO: Add support for testing native apps on BrowserStack and then remove the below code.
            if (!answers.mobile) {
                return;
            }
            logger_1.default.info(ansi_colors_1.default.green('🚀 RUN MOBILE EXAMPLE TESTS ON CLOUD'), '\n');
            if (this.rootDir !== process.cwd()) {
                logger_1.default.info('First, change directory to the root dir of your project:');
                logger_1.default.info(ansi_colors_1.default.cyan(`  cd ${relativeToRootDir}`), '\n');
            }
            const chromeEnvFlag = ' --env browserstack.android.chrome';
            const safariEnvFlag = ' --env browserstack.ios.safari';
            if (answers.runner === Runner.Cucumber) {
                logger_1.default.info('To run your tests with CucumberJS, simply run:');
                logger_1.default.info('  Chrome: ', ansi_colors_1.default.cyan(`${cucumberExample}${chromeEnvFlag}`), '\n');
                logger_1.default.info('  Safari: ', ansi_colors_1.default.cyan(`${cucumberExample}${safariEnvFlag}`), '\n');
            }
            else if (answers.addExamples) {
                if (answers.language === 'ts') {
                    logger_1.default.info('To run an example test (github.ts), run:');
                    logger_1.default.info('  Chrome: ', ansi_colors_1.default.cyan(`${mobileTsExample}${chromeEnvFlag}`), '\n');
                    logger_1.default.info('  Safari: ', ansi_colors_1.default.cyan(`${mobileTsExample}${safariEnvFlag}`), '\n');
                }
                else {
                    logger_1.default.info('To run an example test (ecosia.js), run:');
                    logger_1.default.info('  Chrome: ', ansi_colors_1.default.cyan(`${mobileJsExample}${chromeEnvFlag}`), '\n');
                    logger_1.default.info('  Safari: ', ansi_colors_1.default.cyan(`${mobileJsExample}${safariEnvFlag}`), '\n');
                }
            }
        }
    }
    postConfigInstructions(answers) {
        if (answers.seleniumServer && this.otherInfo.javaNotInstalled) {
            logger_1.default.info('Java Development Kit (minimum v7) is required to run selenium-server locally. Download from here:');
            logger_1.default.info(ansi_colors_1.default.cyan('  https://www.oracle.com/technetwork/java/javase/downloads/index.html'), '\n');
        }
        logger_1.default.info('Happy Testing!');
    }
    pushAnonymousMetrics(answers) {
        var _a;
        const GA_API_KEY = 'XuPojOTwQ6yTO758EV4hBg';
        const GA_TRACKING_ID = 'G-DEKPKZSLXS';
        const payload = {
            'client_id': this.client_id,
            'non_personalized_ads': true,
            'timestamp_micros': new Date().getTime() * 1000,
            'events': {
                'name': 'nw_install',
                'params': {
                    browsers: (_a = answers.browsers) === null || _a === void 0 ? void 0 : _a.join(','),
                    cloudProvider: answers.cloudProvider,
                    language: answers.language,
                    runner: answers.runner,
                    addExample: answers.addExamples
                }
            }
        };
        const data = JSON.stringify(payload);
        const options = {
            hostname: 'www.google-analytics.com',
            port: 443,
            path: `/mp/collect?api_secret=${GA_API_KEY}&measurement_id=${GA_TRACKING_ID}`,
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Content-Length': data.length
            }
        };
        const req = https_1.default.request(options);
        req.write(data);
        req.end();
    }
}
exports.NightwatchInit = NightwatchInit;
